;=====================
;real mode boot loader
;=====================

[BITS 16]
[ORG 0x7C00]


main:
	;BIOS sets dl to contain the right device
	mov [bdrive],dl
	
	;clear screen and set up video mode
	mov al,03h ;80x25, 16color
	mov ah,00
	int 10h
	
	;print message
	mov si,boots
	call bootp


	;reset floppy for reading
	resetd:
		mov ah, 0
		mov dl,[bdrive]

		int 13h
		jc resetd


	;loop until sectors read
	readd:
		mov ax,1000h
		mov es,ax
		mov bx,0 ;ES:BX = 1000:0
			;load kernel to this address

		mov ah,02h ;read sectors
		mov al,18 ;read 18 sectors = 1 track
		mov ch,0 ;cylinder 0
		mov cl,02h ;sector 2 onwards
		mov dh,0 ;head 0
		mov dl,[bdrive]
		
		int 13h
		jc readd

	;stop floppy disk motor
	mov dx,0x3f2
        mov al,0
        out dx,al 

	;jump to kernel
	jmp 1000h:0


;boot loader data
bdrive db 0
boots db 'NawiOS loading kernel.',13,10,0


;print for boot loader
bootp:
	mov ah,0x0E
	mov bh,0x00

	.bootl
 		lodsb
 		or al,al
 		jz .bootldone
 		int 10h
 	jmp .bootl
	.bootldone
	ret




;boot sig
times 510-($-$$) db 0
dw 0xAA55

