;===========================
;START OF ACTUAL KERNEL CODE
;===========================
	start:

	;set registers
	 cli
         mov     ax, 1000h
         mov     ds, ax
         mov     es, ax
         mov     fs, ax
         mov     gs, ax

	;stack to 0x9000:0xFFFE
	mov ax,0x9000
	mov ss,ax
	mov sp, 0XFFFE
	sti

	;read hard-drive a
	resethd:
		mov ah, 0
		mov dl,80h
		int 13h
		jc resethd


	mov si,string4
	call print

	readhd:
		mov ax,3000h
		mov es,ax
		mov bx,0 ;ES:BX = 3000:0
			;load HD to this address

		mov ah,02h ;read sectors
		mov al,60 ;read 60 sectors for 30kB, should be enough :)
		mov ch,0 ;cylinder 0
		mov cl,01h ;sector 1 onwards
		mov dh,0 ;head 0
		mov dl,80h
		
		int 13h
		jc readhd


	mov si, string1
	call print

	.mainloop
		call getinput
		call execute
	jmp .mainloop
	
	.halt
	jmp .halt

execute:
	;get cursor position
	mov ah,3
	int 10h
	sub dh,1
	mov dl,2
	mov ah,2
	int 10h ;put cursor to cmd beginning

	;get ascii
	mov ah,8
	int 10h
	cmp al,' '


	jne .epstart
		mov ah,3
		int 10h
		add dh,1
		mov dl,0
		mov ah,2
		int 10h
		ret
	.epstart

	mov ah,3
	int 10h
	sub dl,1
	mov ah,2
	int 10h

	
	;get HD letter
	mov bx,3000h
	mov ds,bx
	mov si,0
	;3000h:0000

	jmp .skipgth
	;AL=HD CHAR
	;CL=SCREEN CHAR

	.bfchd
	mov ah,3
	mov bh,0
	int 10h
	mov dl,1
	mov ah,2
	int 10h


	.skipgth
	.hdloop
		;get cursor position
		mov ah,3
		mov bh,0
		int 10h
		add dl,1 ;next ascii
		mov ah,2
		int 10h 
		mov ah,8
		int 10h ;get ascii to AL
		mov bl,al ;put to bl

		lodsb
		or al,al		
		jz .hddone ;char is zero, exit
		cmp bl,al
		jne .hexit ;if different, exit
		;mov ah,0x0E
		;mov al,'q'
		;int 10h
	jmp .hdloop
	.hexit
		;loop until 0
		.hsbl
			lodsb
			or al,al
			jz .hsbx ;read next file
		jmp .hsbl
		.hsbx

		lodsb
		lodsb
		lodsb
		lodsb
		lodsb
		or al,al
		jnz .bfchd
	

		;put cursor
		mov bh,0
		mov ah,3
		int 10h
		add dh,1
		mov dl,0
		mov ah,2
		int 10h
	
		;put DS back
		mov ax,1000h
		mov ds,ax

		mov si,string5
		call print

		jmp .hddex
	
	.hddone
	cmp bl,' '
	je .chdp

	cmp bl,al
	jne .hsbx ;if different, exit

	.chdp

	;cursor
	mov bh,0
	mov ah,3
	int 10h
	add dh,1
	mov dl,0
	mov ah,2
	int 10h

	lodsb
	mov ah,al
	lodsb
	;int 10h
	xchg al,ah

	;ax has offset, move to bx
	mov bx,ax

	lodsb
	mov ah,al
	lodsb
	xchg al,ah
	;ax has size in bytes, move to cx
	mov cx,ax

	;put address to source 3000h:offset
	mov ax,3000h
	mov ds,ax
	mov si,bx

	;put address to destination 4000h:0000h
	mov ax,4000h
	mov es,ax
	mov ax,0
	mov di,ax

	;loop size times
	.cploop
		movsb
	loop .cploop

	;set registers
	;cli
        mov     ax, 4000h
        mov     ds, ax
        mov     es, ax
        mov     fs, ax
        mov     gs, ax
	sti

	;jump to: 4000h:0 (program code)
	call 4000h:0

	
	;put DS back
	cli
        mov     ax, 1000h
        mov     ds, ax
        mov     es, ax
        mov     fs, ax
        mov     gs, ax
	sti


	.hddex
	ret


print:

	mov ah,0x0E
	mov bh,0x00

	.printl
		lodsb
 		or al,al
 		jz .printdone
 		int 10h
 	jmp .printl
	.printdone
 	ret

getinput:
	mov si,string2
	call print

	.nextkey
		;wait for keypress and put to al
		mov ah,00h
		int 16h
	



		;if not backspace, skip
		cmp al,8
		jne .skipbs
			.foundbs
			;get cursor position
			mov ah,3
			int 10h

			;if x is 2 then skip
			cmp dl,2
			je .nextkey

			;subtract one from x
			sub dl,1

			;reposition
			mov ah,2
			int 10h

			;print empty
			mov al,0
			mov ah,0x0E
			int 10h

			;position -1
			mov ah,3
			int 10h
			sub dl,1
			mov ah,2
			int 10h

			jmp .nextkey
		.skipbs

		;print letter
		mov ah,0x0E
		int 10h
				

		cmp al,13 ;check for enter
		je .inputdone

		;check for max length
		mov ah,3
		int 10h
		cmp dl,79
		jne .nextkey
		sub dl,1
		mov ah,2
		int 10h
		mov al,0
		mov ah,0x0E
		int 10h
		mov ah,3
		int 10h
		sub dl,1
		mov ah,2
		int 10h
				

	jmp .nextkey
	.inputdone
	
	mov si,string3
	call print
	;mov si,string2
	;call print

	ret

pchar:
	mov ah,0Eh
	int 10h
	ret


clearscreen:
	mov al,03h ;80x25, 16color
	mov ah,00
	int 10h
	ret


;DATA
string1 db 'Kernel loading complete.',13,10,0
string2 db '> ',0
string3 db 13,10,0
string4 db 'Loading hard-drive.',13,10,0
string5 db 'File not found.',13,10,0

times 512 db 0
